@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2002 - 2006 Philip Blundell <philb@gnu.org>
  *               2005 Florian Boor <florian.boor@kernelconcepts.de>
  *
  * This program is free software; you can redistribute it and/or
@@ -88,12 +88,12 @@ handle_size_allocate (GtkWidget *w, GtkAllocation *a)
   int x0, y0, x1, y1;
   int width, height;
 
-  width = a->width + 1;
-  height = a->height + 1;
+  width = a->width;
+  height = a->height;
 
   x0 = y0 = 0;
-  x1 = width - 2;
-  y1 = height - 2;
+  x1 = width - 1;
+  y1 = height - 1;
 
   bitmap = gdk_pixmap_new (NULL, width, height, 1);
   
@@ -127,11 +127,11 @@ handle_size_allocate (GtkWidget *w, GtkAllocation *a)
   /* Top */
   gdk_draw_rectangle (bitmap, one_gc, TRUE, x0 + arc_size, y0, width - arc_size * 2, arc_size);
   /* Bottom */
-  gdk_draw_rectangle (bitmap, one_gc, TRUE, x0 + arc_size, y1 - arc_size, width - arc_size * 2, arc_size);
+  gdk_draw_rectangle (bitmap, one_gc, TRUE, x0 + arc_size, y1 - arc_size, width - arc_size * 2, arc_size + 1);
   /* Left */
   gdk_draw_rectangle (bitmap, one_gc, TRUE, x0, y0 + arc_size, arc_size, height - arc_size * 2);
   /* Right */
-  gdk_draw_rectangle (bitmap, one_gc, TRUE, x1 - arc_size, y0 + arc_size, arc_size, height - arc_size * 2);
+  gdk_draw_rectangle (bitmap, one_gc, TRUE, x1 - arc_size, y0 + arc_size, arc_size + 1, height - arc_size * 2);
 
   g_object_unref (one_gc);
   g_object_unref (zero_gc);
@@ -160,7 +160,7 @@ handle_expose (GtkWidget *w, GdkEventExpose *ev, GtkWidget *child)
   gdk_draw_arc (dr, black_gc, FALSE, x1 - arc_size * 2, y0, arc_size * 2, arc_size * 2, 0*64, 90*64);
   /* South-west corner */
   gdk_draw_arc (dr, black_gc, FALSE, x0, y1 - arc_size * 2, arc_size * 2, arc_size * 2, 180*64, 90*64);
-  /* South-east corner */
+ /* South-east corner */
   gdk_draw_arc (dr, black_gc, FALSE, x1 - arc_size * 2, y1 - arc_size * 2, arc_size * 2, arc_size * 2, 270*64, 90*64);	
 
   /* Join them up */
@@ -183,8 +183,8 @@ popup_box (const gchar *text, gint length, gint x, gint y, gint type)
   GdkGeometry geometry;
   gint spacing = gpe_get_border ();
   gint width = 18;
-  gint height = 18;
-  gint timeout, xsize, lwidth, isize;
+  gint height = 32;
+  gint timeout, xsize, lwidth, iwidth, ysize, lheight, iheight, xpos, ypos;
   PangoLayout *layout;
   GdkColor color;
 
@@ -238,24 +238,40 @@ popup_box (const gchar *text, gint length, gint x, gint y, gint type)
   gtk_widget_realize (label);
  
   /* Calculate size used by the popup.*/ 
-  gtk_label_get_layout_offsets(GTK_LABEL(label), &xsize, NULL);
+  gtk_label_get_layout_offsets(GTK_LABEL(label), &xsize, &ysize);
   layout = gtk_label_get_layout(GTK_LABEL(label));
-  pango_layout_get_pixel_size(layout, &lwidth, NULL); 
+  pango_layout_get_pixel_size(layout, &lwidth, &lheight); 
   
-  gtk_icon_size_lookup(GTK_ICON_SIZE_SMALL_TOOLBAR, &isize, NULL);
-  xsize = xsize + lwidth + 6 + isize + gpe_get_boxspacing() + (spacing * 2);
+  gtk_icon_size_lookup(GTK_ICON_SIZE_SMALL_TOOLBAR, &iwidth, &iheight);
+  xsize = xsize + lwidth + 6 + iwidth + gpe_get_boxspacing() + (spacing * 2);
+  ysize = ysize + lheight + 6 + iheight;
 
   /* Place popup on window according to given coordinates. Default to the top 
      right corner if no coordinates given. */
 	 
   if (x >= 0)
     {
-      geometry.max_width = gdk_screen_width () - spacing - x;
-      geometry.max_height = (gdk_screen_height () - y) / 2;
       if ((gdk_screen_width() < (xsize + x)) && (x > (gdk_screen_width()/2)))
-          gtk_widget_set_uposition (GTK_WIDGET (popup), x - xsize, y);
+        {
+          xpos = x - xsize;
+          geometry.max_width = gdk_screen_width () - spacing - (x - xsize);
+        }
       else
-          gtk_widget_set_uposition (GTK_WIDGET (popup), x, y);
+        {
+          xpos = x;
+          geometry.max_width = gdk_screen_width () - spacing - x;
+        }
+      if ((gdk_screen_height() < (ysize + y)) && (y > (gdk_screen_height()/2)))
+        {
+          ypos = y - ysize;
+          geometry.max_height = gdk_screen_height () / 2;
+        }
+      else
+        {
+          ypos = y;
+          geometry.max_height = gdk_screen_height () / 2;
+        }
+      gtk_widget_set_uposition (GTK_WIDGET (popup), xpos, ypos);
     }
   else
     {
@@ -485,9 +501,13 @@ on_event (XEvent *xevent, GdkEvent * event,
 {
 	if (xevent->type == PropertyNotify)
 	{
-//		printf("property notification (X) %i\n%s\n", event->type, gdk_x11_get_xatom_name_for_display (display, xevent->xproperty.atom));
 		if (xevent->xproperty.atom == gdk_x11_atom_to_xatom(infoprint_atom))
 			do_infoprint_message(NULL, NULL, NULL);
+		if ((xevent->xproperty.atom == gdk_x11_atom_to_xatom(help_atom)) 
+             && (xevent->xproperty.state == PropertyNewValue) 
+             && (xevent->xproperty.window == gdk_x11_drawable_get_xid (dock_window)))
+			do_help_message(NULL, NULL, NULL);
+        return GDK_FILTER_REMOVE;
 	}
 	
   return GDK_FILTER_CONTINUE;
